# Only execute commands if file was not included already
if (NOT CAVESCENEMANAGER_CONFIG_CMAKE_LOADED)

include (${CAVESCENEMANAGER_SOURCE_DIR}/user.cmake)

# Set a default build type for single-configuration
# CMake generators if no build type is set.
if (NOT CMAKE_CONFIGURATION_TYPES AND NOT CMAKE_BUILD_TYPE)
  set (CMAKE_BUILD_TYPE Release)
endif (NOT CMAKE_CONFIGURATION_TYPES AND NOT CMAKE_BUILD_TYPE)

###############################################################################
# Find OpenSG path and set Compiler/Linker flags
###############################################################################
include (${CAVESCENEMANAGER_SOURCE_DIR}/cmake/findOpenSG.cmake)
if (WIN32)
        set (OPENSG_CXX_FLAGS_DEBUG "/D WIN32 /D _LIB /D _DEBUG /D _WINDOWS /D _OSG_HAVE_CONFIGURED_H_ /D OSG_BUILD_DLL /D OSG_DEBUG /D OSG_WITH_GLUT /D OSG_WITH_GIF /D OSG_WITH_TIF /D OSG_WITH_JPG")
        set (OPENSG_CXX_FLAGS_RELEASE "/D WIN32 /D _LIB /D _WINDOWS /D _OSG_HAVE_CONFIGURED_H_ /D OSG_BUILD_DLL /D OSG_WITH_GLUT /D OSG_WITH_GIF /D OSG_WITH_TIF /D OSG_WITH_JPG")
        include_directories (${OPENSG_DIR}/include)
        link_directories (${OPENSG_DIR}/lib)
elseif (APPLE AND ${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
        set (OPENSG_LIBS Base System GLUT)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --dbg --cflags ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_CXX_FLAGS_DEBUG)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --opt --cflags ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_CXX_FLAGS_RELEASE)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --dbg --libs ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_LINKER_FLAGS_DEBUG)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --opt --libs ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_LINKER_FLAGS_RELEASE)
elseif (UNIX)
        set (OPENSG_LIBS Base System GLUT)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --dbg --cflags ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_CXX_FLAGS_DEBUG)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --opt --cflags ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_CXX_FLAGS_RELEASE)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --dbg --libs ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_LINKER_FLAGS_DEBUG)
        exec_program (${OPENSG_BIN_DIR}/${OPENSG_BIN_EXECUTABLE}
                ARGS --opt --libs ${OPENSG_LIBS}
                OUTPUT_VARIABLE OPENSG_LINKER_FLAGS_RELEASE)
else (WIN32)
        message (SEND_ERROR "Could not determine target system")
endif (WIN32)

###############################################################################
# Set Compiler flags
###############################################################################
if (NOT WIN32)
  set (CMAKE_CXX_FLAGS "-fPIC")
  set (CMAKE_CXX_FLAGS_PROFILE "-O3 -g -DNDEBUG -fPIC")
  set (CMAKE_CXX_FLAGS_RELWITHDEBINFO "-O3 -g -DNDEBUG -fPIC")
else (NOT WIN32)
  if (UNIX)
    #mingw, cygwin etc.
    message (FATAL_ERROR "Found unsupported operating system: Only Windows and Linux systems are supported!")  
  else (UNIX)
    #pure win32
    if (MSVC70 OR MSVC71)
      set (CMAKE_CXX_FLAGS "/D _WIN32 /D _WINSOCKAPI_ /Gy /GR /EHsc /W1 /nologo /c /TP /wd4005 /wd4103")
      set (CMAKE_CXX_FLAGS_RELWITHDEBINFO "")
      set (CMAKE_CXX_FLAGS_DEBUG "/Od /ZI /D DEBUG /MDd /Gm /RTC1 /RTCc")
      set (CMAKE_CXX_FLAGS_RELEASE "/Ob2 /Oi /Ot /D NDEBUG /GL /MD")
    elseif (MSVC90)
      set (CMAKE_CXX_FLAGS "/D _WIN32 /D _WINSOCKAPI_ /MP /Gy /EHsc /W1 /nologo /c /TP /wd4005 /wd4103")
      set (CMAKE_CXX_FLAGS_RELWITHDEBINFO "")
      set (CMAKE_CXX_FLAGS_DEBUG "/Od /ZI /D DEBUG /MDd /RTC1 /RTCc /fp:except")
      set (CMAKE_CXX_FLAGS_RELEASE "/Ob2 /Oi /Ot /D NDEBUG /GL /MD /fp:fast")
    else (MSVC70 OR MSVC71)
      set (CMAKE_CXX_FLAGS "/D _WIN32 /D _WINSOCKAPI_ /MP /Gy /EHsc /W1 /nologo /c /TP /wd4005 /wd4103")
      set (CMAKE_CXX_FLAGS_RELWITHDEBINFO "")
      set (CMAKE_CXX_FLAGS_DEBUG "/Od /ZI /D DEBUG /MDd /Gm /RTC1 /RTCc /fp:except")
      set (CMAKE_CXX_FLAGS_RELEASE "/Ob2 /Oi /Ot /D NDEBUG /GL /MD /fp:fast")
    endif (MSVC70 OR MSVC71)
  endif (UNIX)
endif (NOT WIN32)

set (CMAKE_CXX_FLAGS_RELEASE "${CMAKE_CXX_FLAGS_RELEASE} ${OPENSG_CXX_FLAGS_RELEASE}")
set (CMAKE_CXX_FLAGS_DEBUG "${CMAKE_CXX_FLAGS_DEBUG} ${OPENSG_CXX_FLAGS_DEBUG}")
set (CMAKE_SHARED_LINKER_FLAGS_RELEASE "${CMAKE_SHARED_LINKER_FLAGS_RELEASE} ${OPENSG_LINKER_FLAGS_RELEASE}")
set (CMAKE_SHARED_LINKER_FLAGS_DEBUG "${CMAKE_SHARED_LINKER_FLAGS_DEBUG} ${OPENSG_LINKER_FLAGS_DEBUG}")
set (CMAKE_EXE_LINKER_FLAGS_RELEASE "${CMAKE_EXE_LINKER_FLAGS_RELEASE} ${OPENSG_LINKER_FLAGS_RELEASE}")
set (CMAKE_EXE_LINKER_FLAGS_DEBUG "${CMAKE_EXE_LINKER_FLAGS_DEBUG} ${OPENSG_LINKER_FLAGS_DEBUG}")

###############################################################################
# set formatting for eclipse output
###############################################################################
if (CMAKE_COMPILER_IS_GNUCC)
  set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -fmessage-length=0")
endif (CMAKE_COMPILER_IS_GNUCC)
if (CMAKE_COMPILER_IS_GNUCXX)
  set (CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -fmessage-length=0")
endif (CMAKE_COMPILER_IS_GNUCXX)


###############################################################################
# set include directories
###############################################################################
include_directories (${CAVESCENEMANAGER_SOURCE_DIR}/include)

set (CAVESCENEMANAGER_CONFIG_CMAKE_LOADED ON)
endif (NOT CAVESCENEMANAGER_CONFIG_CMAKE_LOADED)

