/*---------------------------------------------------------------------------*\
 *           interactive networked Virtual Reality system (inVRs)            *
 *                                                                           *
 *    Copyright (C) 2005-2009 by the Johannes Kepler University, Linz        *
 *                                                                           *
 *                            www.inVRs.org                                  *
 *                                                                           *
 *              contact: canthes@inVRs.org, rlander@inVRs.org                *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                License                                    *
 *                                                                           *
 * This library is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU Library General Public License as published    *
 * by the Free Software Foundation, version 2.                               *
 *                                                                           *
 * This library is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Library General Public License for more details.                          *
 *                                                                           *
 * You should have received a copy of the GNU Library General Public         *
 * License along with this library; if not, write to the Free Software       *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                        Project: CAVESceneManager                          *
 *                                                                           *
 * The CAVE Scene Manager (CSM) is a wrapper for the OpenSG multi-display    *
 * functionality, which allows the easy configuration of a multi-display     *
 * setup via text files. It was developed by Adrian Haffegee of the Centre   *
 * for Advanced Computing and Emerging Technologies (ACET) at the            *
 * University of Reading, UK as part of his MSc Thesis.                      *
 *                                                                           *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                Changes                                    *
 *                                                                           *
 *                                                                           *
\*---------------------------------------------------------------------------*/


// OpenSG CSM mono server
//
// Mono OpenSG cluster server. In OpenSG
// the terms server and client are used similar to X11. The
// application is the client. Instances that are used for
// rendering are called server.
//
// See the ClusterClient.cpp for an example of how to use them.
//   ./server-mono -geometry 300x300+500+100 -a origin -w front &

// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// ///
// DO NOT CHANGE THIS FILE UNLESS YOU KNOW WHAT YOU ARE DOING !!!
// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// ///

#include <iostream>

// GLUT is used for window handling
#include <OpenSG/OSGGLUT.h>
// General OpenSG configuration, needed everywhere
#include <OpenSG/OSGConfig.h>
// The Cluster server definition
#include <OpenSG/OSGClusterServer.h>
// The GLUT-OpenSG connection class
#include <OpenSG/OSGGLUTWindow.h>
// Render action definition.
#include <OpenSG/OSGRenderAction.h>

OSG_USING_NAMESPACE

// local glut window
GLUTWindowPtr   window;
// render action
RenderAction   *ract;
// pointer the the cluster server instance
ClusterServer  *server;

// forward declarations
void display();
void reshape( int width, int height );


// Initialize GLUT & OpenSG and start the cluster server
int main(int argc,char **argv)
{
    int winid;
    char *name = "ClusterServer";
    char *connectionType = "StreamSock";
    bool fullscreen =true;
    char *address = NULL;

    // evaluate params
    for(int a=1 ; a<argc ; ++a)
    {
        if(argv[a][0] == '-')
        {
            switch(argv[a][1])
            {
                case 'm':
                    connectionType="Multicast";
                    break;
                case 'w':
                    fullscreen=false;
                    break;
                case 'a':
                    address = argv[a][2] ? argv[a]+2 : argv[++a];
                    if(address == argv[argc])
                    {
                        SLOG << "address missing" << endLog;
                        return 0;
                    }
//                    std::cout << address << endLog;
                    break;
                case 'g':
                    //geometry will be caught by glutInit, skip it, + its value
                    if (strcmp(&argv[a][1], "geometry")==0) {
                        a++;
                        break;
                    }
		    printf("\n\nBad geom...\n");
                    // incorrect value falls through...
                case 'd':
                    //display will be caught by glutInit, skip it, + its value
                    if (strcmp(&argv[a][1], "display")==0) {
                        a++;
                        break;
                    }
                    // incorrect value falls through to bomb out...
                default:
                    std::cout << argv[0]
                            << "-m "
                            << "-w "
                            << "-a address "
                            << "-g geometry "
                            << endLog;
                    return 0;
            }
        }
        else
        {
            name=argv[a];
        }
    }
    try
    {
        // initialize Glut
        glutInit(&argc, argv);
        glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );

        // ChangeList needs to be set for OpenSG 1.4
        ChangeList::setReadWriteDefault();
        // init OpenSG
        osgInit(argc, argv);

        winid = glutCreateWindow(name);
        if(fullscreen) {
            glutFullScreen();
        }
        glutDisplayFunc(display);
        glutIdleFunc(display);
        glutReshapeFunc(reshape);
        glutSetCursor(GLUT_CURSOR_NONE);

        glEnable( GL_LIGHTING );
        glEnable( GL_LIGHT0 );
        glEnable( GL_NORMALIZE );

        // create the render action
        ract=RenderAction::create();

        // setup the OpenSG Glut window
        window     = GLUTWindow::create();
        window->setId(winid);
        window->init();

        // create the cluster server
        server     = new ClusterServer(window,name,connectionType,address);
        // start the server
        server->start();

        // enter glut main loop
        glutMainLoop();
    }
    catch(OSG_STDEXCEPTION_NAMESPACE::exception &e)
    {
        SLOG << e.what() << endLog;
        delete server;
        osgExit();
    }
    return 0;
}

// render loop
void display()
{
    try
    {
        // receive scenegraph and do rendering
        server->render(ract);
        // clear changelist
        OSG::Thread::getCurrentChangeList()->clearAll();
    }
    catch(OSG_STDEXCEPTION_NAMESPACE::exception &e)
    {
        SLOG << e.what() << endLog;
        // try to restart server
        server->stop();
        // start server, wait for client to connect
        server->start();
    }
}

// window reshape
void reshape( int width, int height )
{
    // set new window size
	window->resize( width, height );
}
